//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// ecijtpn - ECI_SYNC_TPN and ECI_ASYNC_TPN calls
//           (1) eci_sync_tpn
//           (2) eci_async_tpn
//           (3) eci_get_reply_wait
//           (4) eci_sync
//           (5) eci_async
//           (6) eci_get_reply_wait
//
// Written : Jun 97 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// This program tests the call types: eci_sync_tpn and eci_async_tpn,
// and uses method isTPNTransid to confirm whether tpn is being used.
//
// Note: tpn is currently not supported on Unix platforms.
//
// Testing should include:
// (1) issuing the calls using the default transid CPMI
// (2) issuing the calls specifying your own transid. You must
//     define your own transid in the server. For example, on
//     CICS for OS/2  CPMI runs program FAAMIR; therefore you
//     can define your own transid, eg MYMI, to call program FAAMIR.
//     The example below shows how to override CPMI with MYMI.
//     Note that the value of transid is case-sensitive.
// (3) testing with a mirror less than 4-characters in length.
// (4) with a transid undefined on the server. The _tpn calls should
//     abend, but the following non-tpn calls should use the default
//     CPMI transid and work successfully.
//
// Issuing "java ecijtpn -?" shows the following help and defaults:
//
//       ECIJTPN - ECI_SYNC_TPN and ECI_ASYNC_TPN calls
//       Usage: java ecijtpn <parameters>
//              -j  JGate              Default: (local hostname)
//              -r  JGate port         Default: 2006
//              -s  CICS server        Default:
//              -a  application        Default: SRVTIME
//              -t  transid            Default: CPMI
//              -i  # iterations       Default: 1
//              -d  delay              Default: 0 seconds
//              -u  userid             Default: SYSAD
//              -p  password           Default: SYSAD
//              -!  debug              Default: false
//              -?  This help panel
//
// Example: java ecijtpn -s cicstcp -t MYMI
//
// The above example overrides the server setting of <null> with cicstcp,
// and the default CICS mirror transid of CPMI with MYMI.
//
// Parameter overrides:
//      -j ... hostname or IP address where Java Gateway is running.
//             The default is your local hostname.
//      -p ... Java Gateway port. This defaults to 2006 unless
//             otherwise specified when starting the Java Gateway.
//      -s ... if the CICS Client is being used to connect to a CICS
//             server, null takes the first CICSCLI.INI setting.
//             Otherwise specify a valid entry defined in the CICSCLI.INI
//             file. If a CICS Client is not being used, a null value
//             will use the local server ECI, if supported.
//      -a ... name of server application.
//      -t ... name of transid server program to run under. Equivalent
//             to eci_tpn on (common) CICS Client.
//      -i ... number of iterations (of each set of three calls).
//      -d ... a delay in seconds between each iteration.
//      -u ... userid, if required by the server mirror transaction.
//      -p ... password, if required by the server mirror transaction.
//      -! ... some debug info is supplied if "-!" is specified.
//      -? ... displays the help panel and defaults.
//
// Prior to each ECI call the commarea is initialised to
// 18 dashes. The server returns the current date and timestamp.
// The default CICS server program is the supplied SRVTIME
// which is compatible on all CICS servers. If desired, a different server
// program can be supplied as an input parameter to this applet/application.
// This program contains no data conversion for the commarea.
//
// If run as an applet - parameters are passed via ecijtpn.html.
// If run as an application - parameters can be overridden via command line.
//
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class ecijtpn extends Applet implements Runnable {
   // set up variables
   int iEci_call, iLUW, iCounter, iIt_Count, iReply_Count;
   int     iIterations  = 1;
   int     iPort        = 2006;
   int     iDelay       = 0;
   int     iReplyOK     = 0;
   int     iCallsIssued = 0;
   String  sDebug       = " ";
   String  sWait        = " ";
   String  sGetReply    = "";
   boolean bReply_Rcvd  = false;

   // CICS commarea
   byte[] Commarea = new byte[18];

   // application defaults
   String sServer      = "" ;
   String sUseServer    = "" ;
   String sUserid       = "SYSAD" ;
   String sPassword     = "SYSAD" ;
   String sProgname     = "SRVTIME" ;
   String sTransid      = "CPMI" ;
   boolean bDebug       = false ;
   boolean bWait        = false ;
   String sJGate        = "" ;
   String sPort         = "2006" ;
   String sIterations  = "1";
   String sDelay       = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate      = getParameter("j");
      sPort       = getParameter("r");
      sServer     = getParameter("s");
      sProgname   = getParameter("a");
      sIterations = getParameter("i");
      iIterations = Integer.parseInt(sIterations);
      sUserid     = getParameter("u");
      sPassword   = getParameter("p");
      sDelay      = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay      = Integer.parseInt(sDelay) * 1000;
      sDebug      = getParameter("!");
      // convert to boolean
      if (sDebug.equals("true")) bDebug = true;
      if (sWait.equals("true"))  bWait  = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(10, 130, 380, 260);
      texMessages.reshape(400, 30, 550, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("ECIJTPN", 10, 60);
      g.drawString("ECI_SYNC_TPN and ECI_ASYNC_TPN calls",10, 90);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class ecijtpn
      ecijtpn testApp = new ecijtpn();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               testApp.sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-t") > -1) {
               // set transid
               testApp.sTransid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
            else if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nECIJTPN - ECI_SYNC_TPN and ECI_ASYNC_TPN calls") ;
               testApp.displaySettings("Usage: java ecijtpn <parameters>");
               testApp.displaySettings("       -j  JGate              Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port         Default: "+testApp.sPort);
               testApp.displaySettings("       -s  CICS server        Default: (not specified)");
               testApp.displaySettings("       -a  application        Default: "+testApp.sProgname);
               testApp.displaySettings("       -t  transid            Default: "+testApp.sTransid);
               testApp.displaySettings("       -i  # iterations       Default: "+testApp.sIterations);
               testApp.displaySettings("       -d  delay              Default: "+testApp.sDelay+" seconds");
               testApp.displaySettings("       -u  userid             Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password           Default: "+testApp.sPassword);
               testApp.displaySettings("       -!  debug              Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
         // end FOR loop
         }
      // end-if-args
      }
   // now go and do the work
   testApp.run();
   // end-main
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      displaySettings("\nECIJTPN - ECI_SYNC_TPN and ECI_ASYNC_TPN calls") ;
      displaySettings("       -j  JGate              -> "+sJGate);
      displaySettings("       -r  JGate port         -> "+sPort);
      displaySettings("       -s  CICS server        -> "+sUseServer);
      displaySettings("       -a  application        -> "+sProgname);
      displaySettings("       -t  transid            -> "+sTransid);
      displaySettings("       -i  # iterations       -> "+sIterations);
      displaySettings("       -d  delay              -> "+sDelay+" seconds");
      displaySettings("       -u  userid             -> "+sUserid);
      displaySettings("       -p  password           -> "+sPassword);
      displaySettings("       -!  debug              -> "+bDebug);
      displaySettings("       -?  for help panel");

      // create connection to JGate
      JavaGateway jgaConnection;
      jgaConnection=null;
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // ------------------------------------------------------------------
      // Use extended contructor ECIRequest() for synchronous call
      // ------------------------------------------------------------------

      displayMsg("\n#  Call                Mirror  Commarea Data      Length CICS Rc  Abend");
      displayMsg("-- ------------------  ------  -----------------  ------ -------  -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {

         // output banner every 10 iterations
         if (iIt_Count%10==0) {
            displayMsg("\n#   Call                Mirror  Commarea Data      Length CICS Rc  Abend");
            displayMsg("--  ------------------  ------  -----------------  ------ -------  -----");
         } // end-if

         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC_TPN,
                                                  sServer,
                                                  sUserid,
                                                  sPassword,
                                                  sProgname,
                                                  sTransid,
                                                  Commarea,
                                                  0,
                                                  ECIRequest.ECI_NO_EXTEND,
                                                  ECIRequest.ECI_LUW_NEW );

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            String sDashData  = new String(eciRequest.Commarea,0);

         // Flow synchronous request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_sync_tpn flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         String ComData  = new String(eciRequest.Commarea,0);

         displayMsg(iIt_Count+"  eci_sync_tpn        " + sTransid + "    "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

         // for debug purposes
         GatewayTrace.traceln("Back again : AbendCode = " +
                               eciRequest.Abend_Code);
         GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

         // keep tally of successful return of commarea
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if

         displayMsg(iIt_Count+"  ....isTPNTransid?   "
                             + eciRequest.isTPNTransid());

         // ------------------------------------------------------------------
         // ECI_ASYNC
         // ------------------------------------------------------------------

         eciRequest.Call_Type          = ECIRequest.ECI_ASYNC_TPN;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea,0);

         // flow ECI request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_async_tpn flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea,0);

         // Display result
         displayMsg(iIt_Count+"  eci_async_tpn       " + sTransid + "    "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

         // keep tally of successful call
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if

         displayMsg(iIt_Count+"  ....isTPNTransid?   "
                             + eciRequest.isTPNTransid());

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea,0);

         // flow ECI request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_get _reply_wait ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea,0);

         // Display result
         displayMsg(iIt_Count+"  eci_get_reply_wait          "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code + "\n");

         // keep tally of successful return of commarea
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if

         // Delay next ECI call if delay parameter specified
         try {
            Thread.sleep(iDelay);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during sleep() ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // now without tpn

         eciRequest.Call_Type          = ECIRequest.ECI_SYNC;

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData  = new String(eciRequest.Commarea,0);

         // Flow synchronous request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_sync flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData  = new String(eciRequest.Commarea,0);

         displayMsg(iIt_Count+"  eci_sync            " + sTransid + "    "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

         // for debug purposes
         GatewayTrace.traceln("Back again : AbendCode = " +
                               eciRequest.Abend_Code);
         GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

         // keep tally of successful return of commarea
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if

         displayMsg(iIt_Count+"  ....isTPNTransid?   "
                             + eciRequest.isTPNTransid());

         // ------------------------------------------------------------------
         // ECI_ASYNC
         // ------------------------------------------------------------------

         eciRequest.Call_Type          = ECIRequest.ECI_ASYNC;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea,0);

         // flow ECI request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_async flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea,0);

         // Display result
         displayMsg(iIt_Count+"  eci_async           " + sTransid + "    "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

         // keep tally of successful call
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if

         displayMsg(iIt_Count+"  ....isTPNTransid?   "
                             + eciRequest.isTPNTransid());

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea,0);

         // flow ECI request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_get_reply_wait flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea,0);

         // Display result
         displayMsg(iIt_Count+"  eci_get_reply_wait          "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code + "\n");

         // keep tally of successful return of commarea
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if

         // Delay next ECI call if delay parameter specified
         try {
            Thread.sleep(iDelay);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during sleep() ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count

      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }
      displaySettings("\n--- Calls successful ... "+
                 +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                           + " out of " + iIterations );

   } // end-run() method

} // end-ecijtpn class
