//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// synjms3 - Synchronous Calls to Multiple Servers
//
// Written : July 97 - Alan Hollingshead
//
// This applet/application can issue non-extended ECI synchronous calls
// to one, two, or three servers.
//
// Issuing "java synjms3 -?" shows the following help and defaults:
//
//       SYNJMS3 - Synchronous Calls to Multiple Servers
//       Usage: java synjms3 <parameters>
//              -j  JGate                      Default: (local hostname)
//              -r  JGate port                 Default: 2006
//              -l  list servers               Default: false
//              -s1 CICS server-1              Default: (not specified)
//              -s2 CICS server-2              Default: null
//              -s3 CICS server-3              Default: null
//              -i  # iterations               Default: 1
//              -a  application                Default: SRVTIME
//              -t  transid                    Default: CPMI
//              -u  userid                     Default: SYSAD
//              -p  password                   Default: SYSAD
//              -d  delay between iterations   Default: 0 seconds
//              -!  debug                      Default: false
//              -?  this help panel
//
// Example: java synjms3 -s1 cicsos2 -s2 cicsaix -s3 cicsesa
//
// The above example issues non-extended ECI synchronous calls to
// CICS server-1 (uses default if not specified unless JGate on MVS)
// and then CICS server-2 and server-3 unless these are null.
//
// Parameter overrides:
//      -j ... hostname or IP address where Java Gateway is running.
//             The default is your local hostname.
//      -r ... Java Gateway port. This defaults to 2006 unless
//             otherwise specified when starting the Java Gateway.
//      -l ... list servers only (does not issue ECI synchronous call)
//      -s1 .. if the CICS Client is being used to connect to a CICS
//             server, null takes the first CICSCLI.INI setting.
//             Otherwise specify a valid entry defined in the CICSCLI.INI
//             file. If a CICS Client is not being used, a null value
//             will use the local server ECI, if supported.
//             Note: a null value is not valid for the JGate on MVS.
//      -s2 .. name of the second CICS server to send an ECI request to.
//      -s3 .. name of the third  CICS server to send an ECI request to.
//      -i ... number of iterations (of each set of three calls).
//      -a ... name of server application. This should be the same name
//             for all servers used.
//      -t ... transid. This will pass in the transid for security purposes
//             but will still use the default mirror (eg CPMI). To actually
//             run under a different mirror, eci_sync_tpn with transid
//             specified, is required.
//      -u ... userid, if required by the server mirror transaction.
//      -p ... password, if required by the server mirror transaction.
//      -d ... a delay in seconds between each iteration.
//      -! ... some debug info is supplied if "-!" is specified.
//      -? ... displays the help panel and defaults.
//
//
// If run as an applet - parameters passed via synjms3.html
// If run as an application - parameters can be overridden via command line
// Issue "synjms3 -?" for help and the default settings.
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class synjms3 extends Applet implements Runnable {
   // set up variables
   int iEci_call, iLUW, iCounter, iIt_Count;
   int iNum_LUWs        = 1;
   int iIterations      = 1;
   int iPort            = 2006;
   int iDelay           = 0;
   int iCallsIssued     = 0;
   int iReplyOK         = 0;
   int iComm_Len        = 18;
   String sDebug        = " ";
   String sSpace        = "           ";
   JavaGateway jgaConnection = null;

   // CICS commarea
   byte[] Commarea      = new byte[18];

   // application defaults
   String sServer1      = "" ;
   String sServer2      = "" ;
   String sServer3      = "" ;
   String sUseServer    = "" ;
   String sThisServer   = "" ;
   String sComm_Len     = "" ;
   String sUserid       = "SYSAD" ;
   String sPassword     = "SYSAD" ;
   String sProgname     = "SRVTIME" ;
   String sTransid      = "CPMI" ;
   boolean bDebug       = false ;
   String sJGate        = "" ;
   String sPort         = "2006" ;
   String sExt_mode     = "ECI_EXTENDED";
   String sNum_LUWs     = "1";
   String sIterations   = "1";
   String sDelay        = "0";
   String  sListServers = " ";
   boolean bListServers = false;
   int    iTotalSystems = 0;

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate       = getParameter("j");
      sPort        = getParameter("r");
      sListServers = getParameter("l");
      sServer1     = getParameter("s1");
      sServer2     = getParameter("s2");
      sServer3     = getParameter("s3");
      sProgname    = getParameter("a");
      sTransid     = getParameter("t");
      sIterations  = getParameter("i");
      iIterations  = Integer.parseInt(sIterations);
      sUserid      = getParameter("u");
      sPassword    = getParameter("p");
      sDelay       = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay       = Integer.parseInt(sDelay) * 1000;
      sDebug       = getParameter("!");
      // convert to boolean
      if (sDebug.equals("true"))    bDebug = true;
      if (sListServers.equals("true"))  bListServers = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(10, 110, 420, 320);
      texMessages.reshape(440, 30, 530, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("SYNJMS3", 10, 50);
      g.drawString("Synchronous Calls to Multiple Servers", 10, 80);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class synjms3
      synjms3 testApp = new synjms3();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nSYNJMS3 - Synchronous Calls to Multiple Servers") ;
               testApp.displaySettings("Usage: java synjms3 <parameters>");
               testApp.displaySettings("       -j  JGate                      Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port                 Default: "+testApp.sPort);
               testApp.displaySettings("       -l  list servers               Default: false");
               testApp.displaySettings("       -s1 CICS server1               Default: (not specified)");
               testApp.displaySettings("       -s2 CICS server1               Default: (null)");
               testApp.displaySettings("       -s3 CICS server1               Default: (null)");
               testApp.displaySettings("       -i  # iterations               Default: "+testApp.sIterations);
               testApp.displaySettings("       -a  application                Default: "+testApp.sProgname);
               testApp.displaySettings("       -t  transid                    Default: "+testApp.sTransid);
               testApp.displaySettings("       -u  userid                     Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password                   Default: "+testApp.sPassword);
               testApp.displaySettings("       -d  delay between iterations   Default: "+testApp.sDelay+" seconds");
               testApp.displaySettings("       -!  debug                      Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
            else if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug = true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-l") > -1) {
               // set bListServers to true
               testApp.bListServers = true ;
            }
            else if (args[loop].indexOf("-s1") > -1) {
               // set cics server1
               testApp.sServer1 = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-s2") > -1) {
               // set cics server2
               testApp.sServer2 = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-s3") > -1) {
               // set cics server3
               testApp.sServer3 = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-t") > -1) {
               // set transid
               testApp.sTransid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
         // end FOR loop
         }
      }
   // now go and do the work
   testApp.run();
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer1.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer1;
      }
      displaySettings("\nSYNJMS3 - Synchronous Calls to Multiple Servers") ;
      displaySettings("       -j  JGate                      -> "+sJGate);
      displaySettings("       -r  JGate port                 -> "+sPort);
      displaySettings("       -l  list servers               -> "+bListServers);
      displaySettings("       -s1 CICS server1               -> "+sUseServer);
      displaySettings("       -s2 CICS server2               -> "+sServer2);
      displaySettings("       -s3 CICS server3               -> "+sServer3);
      displaySettings("       -i  # iterations               -> "+sIterations);
      displaySettings("       -a  application                -> "+sProgname);
      displaySettings("       -t  transid                    -> "+sTransid);
      displaySettings("       -u  userid                     -> "+sUserid);
      displaySettings("       -p  password                   -> "+sPassword);
      displaySettings("       -d  delay between iterations   -> "+sDelay+" seconds");
      displaySettings("       -!  debug                      -> "+bDebug);
      displaySettings("       -?  help panel");

      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if list servers option selected, then listSystems and then end.
      if (bListServers) {
         try {
            listServers();
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error returned within listServers() ---\n");
            displayMsg("--- Caught Exception: " + e);
            return;
         }
         // ------------------------------------------------------------------
         // Close connection to JGate
         // ------------------------------------------------------------------
         try {
            jgaConnection.close();
            displayMsg("\n--- Successfully closed JavaGateway connection ---");
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error closing JavaGateway connection ---\n");
            displayMsg("--- Caught Exception: " + e);
         }
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer1.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         if (eciRequest.Cics_Rc != ECIRequest.ECI_NO_ERROR &&
             eciRequest.Cics_Rc != ECIRequest.ECI_ERR_MORE_SYSTEMS ) {
            displayMsg("\nECIRequest.listSystems failed with Rc : " +
                        eciRequest.getRc());
            try {
               jgaConnection.close();
               displayMsg("\n\n--- Successfully closed JavaGateway connection ---");
            }
            catch (Exception e) {
               GatewayTrace.traceln("ERROR: Caught Exception: " + e);
               displayMsg("\n--- Error closing JavaGateway connection ---\n");
               displayMsg("--- Caught Exception: " + e);
            }
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- CICS server1 : "+ eciRequest.SystemList.elementAt(0));
         sServer1 = (String) eciRequest.SystemList.elementAt(0);
      }


      String sGap1 = sSpace.substring(1,(12-(String.valueOf(iIterations).length())));
      displayMsg("\nIteration  Server    Commarea Data       CICS Rc   Abend");
      displayMsg(  "---------  --------  -----------------   -------   -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {

         // output banner every 15 iterations
         if (iIt_Count%15==0) {
            displayMsg("\nIteration  Server    Commarea Data       CICS Rc   Abend");
            displayMsg(  "---------  --------  -----------------   -------   -----");
         } // end-if

         try {
            // flow ECI request
            sThisServer = sServer1;
            eciSyncCall(sThisServer);

            // flow ECI request to Server2 if specified
            if (sServer2 != "") {
               sThisServer = sServer2;
               eciSyncCall(sThisServer);
            }

            // flow ECI request to Server3 if specified
            if (sServer3 != "") {
               sThisServer = sServer3;
               eciSyncCall(sThisServer);
            }

         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error returned from JGate flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Delay next iteration if delay parameter specified
         try {
            Thread.sleep(iDelay);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during sleep() ---\n");
            displayMsg("--- Caught Exception: " + e);
            return;
         }
         // end iIt_Count for-loop
      }
      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }
      displaySettings("\n--- Calls successful ... "+
                         +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                         + " out of " + iIterations );
   }
   //----------------------------------------------------------------------
   // Method : eciSyncCall
   // Issue an ECI_SYNC call for this server
   //----------------------------------------------------------------------

   private void eciSyncCall(String sThisServer) throws Exception {

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                               sThisServer,
                                               sUserid,
                                               sPassword,
                                               sProgname,
                                               sTransid,
                                               Commarea,
                                               iComm_Len,
                                               ECIRequest.ECI_NO_EXTEND,
                                               ECIRequest.ECI_LUW_NEW );

      // set commarea to dashes
      for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
         // reset commarea to dashes
         eciRequest.Commarea[iCounter]=45;

      iCallsIssued++;
      jgaConnection.flow(eciRequest);

      // for debug purposes
      GatewayTrace.traceln("Back again : AbendCode = " +
                            eciRequest.Abend_Code);
      GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

      // Convert ASCII bytes to ASCII characters
      String ComData  = new String(eciRequest.Commarea,0);

      String sGap2 = sSpace.substring(1,(12-(String.valueOf(iIt_Count).length())));
      String sGap3 = sSpace.substring(1,(9-(sThisServer.length())));
      // Display result
      displayMsg(iIt_Count + sGap2 +
           sThisServer + sGap3 + "  " +
           ComData.substring(0,18)+"  "+
           eciRequest.Cics_Rc +"        "+
           eciRequest.Abend_Code +"    ");

      if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
         iReplyOK++;
      }
   }
   //----------------------------------------------------------------------
   // Method : listServers
   // List servers in CICSCLI.INI file
   //----------------------------------------------------------------------

   private void listServers() throws Exception {

      ECIRequest eciRequest = null;
      eciRequest = ECIRequest.listSystems(99);
      jgaConnection.flow(eciRequest);

      iTotalSystems = (eciRequest.SystemList.size()/2);

      displayMsg("\n--- List of servers available: ");

      if (eciRequest.getRc() == 0) {
          for (int iSysLoop = 0, iSysNum = 1;
                   iSysLoop < eciRequest.SystemList.size();
                   iSysLoop += 2,
                   iSysNum  += 1) {

             displayMsg("     Server ("+iSysNum+") " +
                   (String) eciRequest.SystemList.elementAt(iSysLoop) +
                   ", Description: " +
                   (String) eciRequest.SystemList.elementAt(iSysLoop + 1));
          }
      }
      else
      {
         displayMsg("ECIRequest.listSystems failed with Rc : " +
                     eciRequest.getRc());
         return;
      }
   } // end listServers method
}  // end synjms3 class
