//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// synjtpn - Extended ECI_SYNC_TPN Calls
//
// Written : Jun 97 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// This program tests the call type eci_sync_tpn for extended eci calls
// and multiple LUWs.
//
// The program is based on synj002 which uses eci_sync calls instead.
//
// Note: tpn is currently not supported on Unix platforms.
//
// Testing should include:
// (1) issuing the calls using the default transid CPMI
// (2) issuing the calls specifying your own transid. You must
//     define your own transid in the server. For example, on
//     CICS for OS/2  CPMI runs program FAAMIR; therefore you
//     can define your own transid, eg MYMI, to call program FAAMIR.
//     The example below shows how to override CPMI with MYMI.
//     Note that the value of transid is case-sensitive.
// (3) testing with a mirror less than 4-characters in length.
// (4) with a transid undefined on the server. The _tpn calls should
//     abend, but the following non-tpn calls should use the default
//     CPMI transid and work successfully.
//
// synjtpn is a Java program that on each requested iteration
// issues sets of six ECI synchronous calls for each LUW. The first
// two calls are followed by an ECI_BACKOUT. The fourth and fifth
// calls are followed by an ECI_COMMIT. The commarea on each call
// is initialised to 18 dashes. The server returns the current date
// and timestamp. The default CICS server program is the supplied SRVTIME
// which is compatible on all CICS servers. If desired, a different server
// program can be supplied as an input parameter to this applet/application.
// This program contains no data conversion for the commarea.
//
// If run as an applet - parameters passed via synjtpn.html
// If run as an application - parameters can be overridden via command line
// Issue "synjtpn -?" for help and the default settings.
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class synjtpn extends Applet implements Runnable {
   // set up variables
   int iEci_call, iLUW, iCounter, iIt_Count;
   int iNum_LUWs      = 1;
   int iIterations    = 1;
   int iPort          = 2006;
   int iDelay         = 0;
   int iCallsIssued   = 0;
   int iReplyOK       = 0;
   String sDebug      = " ";

   // CICS commarea
   byte[] Commarea = new byte[18];

   // application defaults
   String sServer     = "" ;
   String sUseServer  = "" ;
   String sUserid     = "SYSAD" ;
   String sPassword   = "SYSAD" ;
   String sProgname   = "SRVTIME" ;
   String sTransid    = "CPMI" ;
   boolean Debug      = false ;
   String sJGate      = "" ;
   String sPort       = "2006" ;
   String sExt_mode   = "ECI_EXTENDED";
   String sNum_LUWs   = "1";
   String sIterations = "1";
   String sDelay      = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate      = getParameter("j");
      sPort       = getParameter("r");
      sServer     = getParameter("s");
      sProgname   = getParameter("a");
      sTransid    = getParameter("t");
      sNum_LUWs   = getParameter("l");
      iNum_LUWs   = Integer.parseInt(sNum_LUWs);
      sIterations = getParameter("i");
      iIterations = Integer.parseInt(sIterations);
      sUserid     = getParameter("u");
      sPassword   = getParameter("p");
      sDelay      = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay      = Integer.parseInt(sDelay) * 1000;
      sDebug      = getParameter("!");
      // convert to boolean
      if (sDebug.equals("true")) Debug = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(10, 130, 380, 260);
      texMessages.reshape(400, 30, 550, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("SYNJTPN", 10, 50);
      g.drawString("Extended ECI_SYNC_TPN Calls", 10, 80);
      g.drawString("Incorporating ECI_BACKOUT and ECI_COMMIT", 10, 110);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class synjtpn
      synjtpn testApp = new synjtpn();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.Debug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               testApp.sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-t") > -1) {
               // set transid
               testApp.sTransid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-l") > -1) {
               // set Num_LUWs
               testApp.sNum_LUWs = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iNum_LUWs = Integer.parseInt(testApp.sNum_LUWs);
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
            else if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nSYNJTPN - Extended ECI_SYNC_TPN Calls") ;
               testApp.displaySettings("Usage: java synjtpn <parameters>");
               testApp.displaySettings("       -j  JGate              Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port         Default: "+testApp.sPort);
               testApp.displaySettings("       -s  CICS server        Default: (not specified)");
               testApp.displaySettings("       -a  application        Default: "+testApp.sProgname);
               testApp.displaySettings("       -t  transid            Default: "+testApp.sTransid);
               testApp.displaySettings("       -l  # concurrent luws  Default: "+testApp.sNum_LUWs);
               testApp.displaySettings("       -i  # iterations       Default: "+testApp.sIterations);
               testApp.displaySettings("       -u  userid             Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password           Default: "+testApp.sPassword);
               testApp.displaySettings("       -d  delay              Default: "+testApp.sDelay+" seconds");
               //    testApp.displaySettings("       -t  eci_tpn parameter  Default: CPMI");
               testApp.displaySettings("       -!  debug              Default: "+testApp.Debug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
         // end FOR loop
         }
      }
   // now go and do the work
   testApp.run();
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      displaySettings("\nSYNJTPN - Extended ECI_SYNC_TPN Calls") ;
      displaySettings("       -j  JGate              -> "+sJGate);
      displaySettings("       -r  JGate port         -> "+sPort);
      displaySettings("       -s  CICS server        -> "+sUseServer);
      displaySettings("       -a  application        -> "+sProgname);
      displaySettings("       -t  transid            -> "+sTransid);
      displaySettings("       -l  # concurrent luws  -> "+sNum_LUWs);
      displaySettings("       -i  # iterations       -> "+sIterations);
      displaySettings("       -u  userid             -> "+sUserid);
      displaySettings("       -p  password           -> "+sPassword);
      displaySettings("       -d  delay              -> "+sDelay+" seconds");
      displaySettings("       -!  debug              -> "+Debug);
      displaySettings("       -?  for help panel");
      // create connection to JGate
      JavaGateway jgaConnection;
      jgaConnection=null;
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // ------------------------------------------------------------------
      // Extended Contructor ECIRequest()
      // ------------------------------------------------------------------

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC_TPN,
                                               sServer,
                                               sUserid,
                                               sPassword,
                                               sProgname,
                                               sTransid,
                                               Commarea,
                                               0,
                                               0,
                                               0);

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {
         displayMsg("\n--- Iteration: "+iIt_Count+" of "+iIterations+
                    " using ECI_SYNC_TPN calls\n");
         displayMsg("# LUW Mirror Extend-Mode  Commarea Data      Length CICS Rc  Abend  LUW Token");
         displayMsg("- --- ------ -----------  -------------      ------ -------  -----  ---------");
         // array for LUW tokens
         int[] aiTokens = new int[iNum_LUWs+1];
         for (iLUW=1; iLUW<=iNum_LUWs; iLUW++)
             // initialise Tokens to zero
             aiTokens[iLUW]=0;

         for (iEci_call=1; iEci_call<=6; iEci_call++) {
            switch(iEci_call) {
               case 3   : eciRequest.Extend_Mode=ECIRequest.ECI_BACKOUT;
                          sExt_mode = "ECI_BACKOUT ";
                          break;
               case 6   : eciRequest.Extend_Mode=ECIRequest.ECI_COMMIT;
                          sExt_mode = "ECI_COMMIT  ";
                          break;
               default  : eciRequest.Extend_Mode=ECIRequest.ECI_EXTENDED;
                          sExt_mode = "ECI_EXTENDED";
                          break;
            }
            // for each LUW
            for (iLUW=1; iLUW<=iNum_LUWs; iLUW++)
               try {
                  // set commarea to dashes
                  for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
                     // reset commarea to dashes
                     eciRequest.Commarea[iCounter]=45;

                  // use correct LUW token
                  eciRequest.Luw_Token=aiTokens[iLUW];

                  // flow ECI request
                  iCallsIssued++;
                  jgaConnection.flow(eciRequest);

                  if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
                     iReplyOK++;

                  // for debug purposes
                  GatewayTrace.traceln("Back again : AbendCode = " +
                                        eciRequest.Abend_Code);
                  GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

                  // Convert ASCII bytes to ASCII characters
                  String ComData  = new String(eciRequest.Commarea,0);

                  // Display result
                  displayMsg(iEci_call+"  "+iLUW+"  "+sTransid+"   "+
                       sExt_mode+" "+
                       ComData.substring(0,eciRequest.Commarea_Length-1)+"  "+
                       eciRequest.Commarea_Length +"     "+
                       eciRequest.Cics_Rc +"       "+
                       eciRequest.Abend_Code +"    "+
                       eciRequest.Luw_Token);

                  // Assign new LUW token
                  aiTokens[iLUW]=eciRequest.Luw_Token;

                  // Delay next ECI call if delay parameter specified
                  try {
                     Thread.sleep(iDelay);
                  }
                  catch (Exception e) {
                     GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                     System.out.println("\n--- Error during sleep() ---\n");
                     System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
                  }
               }  //end LUW for-loop "try"
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  System.out.println("\n--- Error returned from JGate flow ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
         // end iEci_call for-loop
         }
      // end iIt_Count for-loop
      }
      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         System.out.println("\n--- Error closing JavaGateway connection ---\n");
         System.out.println("--- Caught Exception: " + e);
      }
      displaySettings("\n--- Calls successful ... "+
                         +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                         + " out of " + iIterations );
   }
}
